"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetUser = exports.DeleteUserById = exports.DeleteUser = exports.SaveUser = exports.UserRedis = void 0;
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function UserRedis(app) {
    redisClient = app.get('redisClient');
}
exports.UserRedis = UserRedis;
function SaveUser(vcc, user) {
    if (!user || !user.id)
        throw 'CANNOT_SAVE_WITH_NULL_ID';
    return redisClient.hsetAsync(vcc + ':User', user.id, JSON.stringify(user))
        .then(function (result) {
        return redisClient.expireAsync(vcc + ':User', redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveUser = SaveUser;
function DeleteUser(vcc, user) {
    if (!user || !user.id)
        throw 'CANNOT_DELETE_WITH_NULL_ID';
    return redisClient.hdelAsync(vcc + ':User', user.id)
        .then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteUser = DeleteUser;
function DeleteUserById(vcc, userId) {
    if (!userId)
        throw 'CANNOT_DELETE_WITH_NULL_ID';
    return redisClient.hdelAsync(vcc + ':User', userId)
        .then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteUserById = DeleteUserById;
function GetUser(vcc, userId) {
    if (!userId)
        throw 'CANNOT_GET_WITH_NULL_ID';
    return redisClient.hgetAsync(vcc + ':User', userId)
        .then(function (result) {
        var user;
        try {
            user = JSON.parse(result);
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(user);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUser = GetUser;
