"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateI6Interaction = exports.GetChannelOldInteractionSummaryPromises = exports.GetChannelNewInteractionSummaryPromises = exports.GetProcessFilterFromInteractionSearch = exports.GetInteractionSearchFromRequest = exports.GetLocalInteractionFileContent = exports.GetRemoteInteractionFileContent = exports.GetRemoteInteractionFile = exports.InteractionService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var request = require("request");
var ExternalDB = require("../db/external");
var interaction_model_1 = require("../models/interaction.model");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var arrays_shared_1 = require("../shared/arrays.shared");
var http_request_occ_1 = require("../shared/http-request-occ");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var bluebird_1 = require("bluebird");
var integration_config_1 = require("../config/integration.config");
var mssql_config_1 = require("../config/mssql.config");
var process_model_1 = require("../models/process.model");
var audio_transfer_implementation_1 = require("../implementation/audio-transfer.implementation");
var file_manager_1 = require("../shared/file-manager");
var semantic_analysis_implementation_1 = require("../implementation/semantic-analysis.implementation");
var role_service_1 = require("./role.service");
var logger = log4js.getLogger('ServerScripts');
function InteractionService(app) {
    var _this = this;
    app.get('/api/interaction/get_interaction/:evaluationPlanId/:type/:id?', function (req, res, next) {
        logger.info('[InteractionService::get_interaction] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var evaluationPlanId = req.params.evaluationPlanId || '';
        var type = req.params.type || '';
        var id = req.params.id || '';
        var environmentId = null;
        var environmentType = null;
        var interaction = null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!evaluationPlanId) {
                logger.error('[EvaluationPlanService::get_evaluationplan] Received evaluation plan ID is null');
                throw 'SERVER_ERROR_EVALUATION_PLAN_ID_MISSING';
            }
            logger.info('[EvaluationPlanService::get_evaluationplan] Sending request to SP GetEvaluationPlan for EP ID: ' + evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_EVALUATION_PLAN_NOT_FOUND';
            environmentId = result.environmentId;
            environmentType = result.environmentType;
            if (environmentType && (environmentType !== 'allegro' && environmentType !== 'i6')) {
                logger.info('[InteractionService::get_interaction] Requesting environment data');
                return mssql_service_1.MSSql.GetEnvironment(vcc, environmentId);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (result) {
            if (environmentType && (environmentType !== 'allegro' && environmentType !== 'i6')) {
                if (!result)
                    throw 'SERVER_ERROR_ENVIRONMENT_NOT_FOUND';
                environmentId = result.name;
                environmentType = result.type;
            }
            return GetEvaluationPlanInteraction(vcc, type, environmentType, evaluationPlanId, id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_INTERACTION_NOT_FOUND';
            interaction = result;
            if (type === 'CALL' && environmentType && environmentType === 'allegro') {
                fillInteractionContentURL(interaction, vcc);
            }
            return interaction;
        })
            .then(function (result) {
            if (type === 'CALL') {
                if (environmentType && environmentType === 'allegro') {
                    return ExternalDB.inconcert.getCampaignWebdesignerURL(vcc, result.idCampaign, type).then(function (url) {
                        result.managementFormURL = (url && url.length > 0) ? config_1.INCONCERT_APPS_BASE_URL + 'webdesigner/' + url + '?INTERACTIONID=' + result.id + '&AGENTID=' + result.idAgent + '&VIRTUALCC=' + vcc + '&OUTBOUNDPROCESS=&OUTBOUNDBATCH=&SOURCE=QUALITY' : '';
                        return Promise.resolve(result);
                    });
                }
                else {
                    return Promise.resolve(result);
                }
            }
            else {
                return result;
            }
        })
            .then(function (result) {
            if (type === 'CALL') {
                var i_1 = result;
                return mssql_service_1.MSSql.GetAllTagsInteraction(vcc, id).then(function (result) {
                    i_1.tags = result;
                    return Promise.resolve(i_1);
                }).catch(function (err) {
                    i_1.tags = [];
                    return Promise.resolve(i_1);
                });
            }
            else {
                return result;
            }
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[InteractionService::get_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_not_evaluated_interactions_auditor/:idAuditor', function (req, res, next) {
        logger.info('[InteractionService::get_not_evaluated_interactions_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userRole = res.locals.user.role;
        var userId = res.locals.userId;
        var params = new interaction_model_1.InteractionSearch;
        var evaluationsToUpdate = [];
        params.campaigns = req.body.campaigns || [];
        params.agents = req.body.agents || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.types = req.body.types || [];
        params.results = req.body.results ? req.body.results.map(function (r) {
            var dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.dateFin = req.body.dateLimit || '';
        params.evaluationStatus = req.body.evaluationStatus || [true];
        params.interactionIds = req.body.interactionIds || [];
        logger.debug('[InteractionService::get_not_evaluated_interactions_auditor] Received params: :' + JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return role_service_1.GetRoleFunctions(vcc, userId, userRole);
        })
            .then(function (result) {
            if (result && result.data && result.data.hasAdminFunctions)
                params.evaluationStatus.push(false);
            return redis_service_1.Redis.GetEvaluationsStatus(vcc);
        })
            .then(function (result) {
            _this.evaluationsToUpdate = result ? result : [];
            logger.info('[InteractionService::get_not_evaluated_interactions_auditor] Requesting interactions(SP GetNotEvaluatedInteractionsAuditor) for VCC|idAuditor: ' + vcc + '|' + userId);
            return mssql_service_1.MSSql.GetNotEvaluatedInteractionsAuditor(vcc, userId, params.campaigns, params.agents, params.evaluationPlans, params.types, params.results, params.dateFin, params.evaluationStatus.length && params.evaluationStatus.length > 1 ? null : params.evaluationStatus, params.interactionIds.toString());
        })
            .then(function (result) {
            result.map(function (r) {
                r.isEvaluationBeingEvaluated = _this.evaluationsToUpdate.find(function (etu) { return etu.evaluationPlanId === r.idEvaluationPlan && etu.interactionId === r.id && etu.auditorId === r.idAuditor; }) ? true : false;
                r.isInteractionBeingEvaluated = _this.evaluationsToUpdate.find(function (etu) { return etu.evaluationPlanId === r.idEvaluationPlan && etu.interactionId === r.id; }) ? true : false;
            });
            if (res.headersSent)
                return;
            logger.debug('[InteractionService::get_not_evaluated_interactions_auditor] Received response from SP GetNotEvaluatedInteractionsAuditor: ' + JSON.stringify(result.length));
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[InteractionService::get_not_evaluated_interactions_auditor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_evaluated_interactions_auditor/', function (req, res, next) {
        logger.info('[InteractionService::get_evaluated_interactions_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idAuditor = res.locals.userId;
        logger.info('[InteractionService::get_evaluated_interactions_auditor] Filling the filter\'s object');
        var params = new interaction_model_1.InteractionSearch;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results || [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        logger.debug('[InteractionService::get_evaluated_interactions_auditor] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[InteractionService::get_evaluated_interactions_auditor] Calling GetEvaluations SP for VCC|IdAuditor: ' + instance + '|' + idAuditor);
            return mssql_service_1.MSSql.GetEvaluatedInteractionsByAuditor(instance, idAuditor, params.agents.toString(), params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.interactionIds.toString());
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.debug('[InteractionService::get_evaluated_interactions_auditor] Received result: ' + JSON.stringify(result));
            logger.info('[InteractionService::get_evaluated_interactions_auditor] Sending results: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            (function (err) {
                var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
                var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
                logger.error('[InteractionService::get_evaluated_interactions_auditor] Error on \'' + req.path + '\': ' + errorMsg);
                if (res.headersSent)
                    return;
                res.json({ status: false, description: msg, error: errorMsg });
            });
        });
    });
    app.get('/api/interaction/get_evaluated_interactions/:idAgent?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationDateFrom = req.query.evaluationDateFrom || '1980-01-01';
        var evaluationDateTo = req.query.evaluationDateTo || '2050-01-01';
        var idAgent = req.params.idAgent || '';
        var campaigns = req.query.campaigns || 'GetEvaluatedInteractions';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEvaluatedInteractions(instance, idAgent, campaigns, evaluationDateFrom, evaluationDateTo);
        }).then(function (result) {
            var interactions = result;
            var p = [];
            interactions.forEach(function (i) {
                p.push(ExternalDB.interactions.getInteractionByTypeAndId(instance, i.interactionType, i.id)
                    .then(function (r) {
                    i.address = r.address;
                    i.dispositionCode = r.dispositionCode;
                    i.origin = r.origin;
                    i.hasVideo = r.hasVideo;
                    return i;
                }).catch(function (err) {
                    var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                    logger.error('Error obteniendo info de las interacciones: ' + errorMsg);
                    throw err;
                }));
            });
            return Promise.all(p).then(function (values) {
                return [].concat.apply([], values);
            });
        }).then(function (result) {
            var interactions = result || [];
            return fillInteractionTags(instance, interactions);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_evaluated_interactions_by_campaigns_auditors_and_agents?', function (req, res, next) {
        logger.info('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        logger.info('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Filling the filter\'s object');
        var params = new interaction_model_1.InteractionSearch;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.auditors = req.body.auditors || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results || [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        logger.debug('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.debug('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Calling GetEvaluatedInteractionsByCampaignsAuditorsAndAgents SP');
            return mssql_service_1.MSSql.GetEvaluatedInteractionsByCampaignsAuditorsAndAgents(instance, params.auditors.toString(), params.agents.toString(), params.campaigns.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString());
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.debug('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Results from GetEvaluatedInteractionsByCampaignsAuditorsAndAgents SP: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_evaluated_interactions_agent?', function (req, res, next) {
        logger.info('[InteractionService::get_evaluated_interactions_agent] Received ' + req.method + ' on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idAgent = res.locals.userId;
        logger.info('[InteractionService::get_evaluated_interactions_agent] Filling the filter\'s object');
        var params = new interaction_model_1.InteractionSearch;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results || [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        logger.debug('[InteractionService::get_evaluated_interactions_agent] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[InteractionService::get_evaluated_interactions_agent] Calling GetEvaluatedInteractions SP for VCC|idAgent: ' + instance + '|' + idAgent);
            return mssql_service_1.MSSql.GetEvaluatedInteractionsByAuditor(instance, null, idAgent, params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.interactionIds.toString());
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[InteractionService::get_evaluated_interactions_agent] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.get('/api/interaction/get_interactions_evaluationplan/:idEvaluationPlan/:idAuditor?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idEvaluationPlan = req.params.idEvaluationPlan || '';
        var idAuditor = req.params.idAuditor || '';
        var interactions;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(instance, idEvaluationPlan, idAuditor);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_interactions', function (req, res, next) {
        logger.info('[InteractionService::get_interactions] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var type = res.locals.type;
        var userId = res.locals.userId;
        var limit = 10;
        var params = new interaction_model_1.InteractionSearch;
        params = GetInteractionSearchFromRequest(req.body.evaluationFormatId, req.body, false);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                logger.info('[InteractionService::get_interactions] Searching evaluation format\'s campaigns');
                return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(instance, params.evaluationFormatId);
            }
            else {
                return bluebird_1.resolve(true);
            }
        }).then(function (result) {
            if (params.environmentType && (params.environmentType === 'allegro' || params.environmentType === 'i6')) {
                if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                    if (!result || result.length === 0)
                        throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                    params.campaigns = result.map(function (r) { return r.id; });
                }
            }
            return Promise.resolve(true);
        }).then(function (result) {
            if (params.environmentType && (params.environmentType === 'allegro' || params.environmentType === 'i6')) {
                return Promise.resolve(true);
            }
            else {
                var ftpCSVRedisKey = void 0;
                if (params.evaluationFormatId)
                    ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
                else
                    ftpCSVRedisKey = 'FTP_PROCESS:';
                return redis_service_1.Redis.GetCsvProcess(instance, ftpCSVRedisKey + userId);
            }
        }).then(function (result) {
            if (params.environmentType !== 'allegro' && params.environmentType !== 'i6')
                if (result === null)
                    throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
            var promiseArray = [];
            logger.info('[InteractionService::get_interactions] Requesting not evaluated interactions from HistoricalData');
            promiseArray = GetChannelNewInteractionSummaryPromises(instance, type, true, 'SUMMARIZED', params, result.data);
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::get_interactions] Received interactions from database');
            res.json({
                status: true,
                description: 'SERVER_SUCCESS_MESSAGE',
                data: result
            });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::get_interactions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_interaction_view', function (req, res, next) {
        logger.info('[InteractionService::get_interaction_view] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var evaluationPlanId = null;
        var processId = null;
        var interactionType = null;
        var folder = null;
        var campaign = null;
        var interactionId = null;
        var fileType = interactionType === 'CALL' ? 'json' : 'view';
        var interactionItems = [];
        var viewResults = [];
        var transcriptionResults = [];
        var transcriptionQualification = null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (req.body) {
                evaluationPlanId = req.body.evaluationPlanId || null;
                processId = req.body.processId || null;
                interactionType = req.body.interactionType || null;
                folder = req.body.folder || null;
                campaign = req.body.campaign || null;
                interactionId = req.body.interactionId || null;
            }
            else {
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            }
            logger.info('[InteractionService::get_interaction_view] Searching inetraction instances');
            return mssql_service_1.MSSql.GetInteractionTranscription(vcc, evaluationPlanId, processId, interactionType, interactionId);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_INTERACTION_NOT_FOUND';
            interactionItems = folder ? result.filter(function (r) { return r.folder === folder; }) : result;
            var audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
            var promiseArray = [];
            interactionItems.map(function (i) {
                var filePath = '';
                if (i.interactionType === 'MAIL') {
                    for (var n = 1; n <= i.totalMessages; n++) {
                        filePath = audioTransfer.GetFileNamePathFromInteraction(i, 'remote', fileType, n);
                        promiseArray.push(GetRemoteInteractionFile(filePath, i.folder));
                    }
                }
                else {
                    filePath = audioTransfer.GetFileNamePathFromInteraction(i, 'remote', fileType);
                    promiseArray.push(GetRemoteInteractionFile(filePath, i.folder));
                }
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            viewResults = result;
            var semanticAnalysis = new semantic_analysis_implementation_1.SemanticAnalysisImplementation();
            var promiseArray = [];
            interactionItems.map(function (i) {
                promiseArray.push(semanticAnalysis.SemanticAnalysisViewFileGenerate(i));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            transcriptionResults = result;
            var transcriptionDTO = {
                vcc: vcc,
                processId: processId,
                id: interactionId,
                section: null,
                part: null
            };
            transcriptionDTO.interactionType = interactionType;
            transcriptionDTO.folder = folder;
            transcriptionDTO.campaign = campaign;
            return mssql_service_1.MSSql.GetTranscription(transcriptionDTO);
        }).then(function (result) {
            transcriptionQualification = (result && result.StatusCodeError) ? null : result;
            if (res.headersSent)
                return;
            logger.info('[InteractionService::get_interaction_view] Received results');
            res.json({
                status: true,
                description: 'SERVER_SUCCESS_MESSAGE',
                data: {
                    interactionView: viewResults,
                    transcriptionResult: transcriptionResults,
                    transcriptionQualification: transcriptionQualification && transcriptionQualification.value ? transcriptionQualification.value : null
                }
            });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::get_interaction_view] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_interaction_query_results', function (req, res, next) {
        logger.info('[InteractionService::get_interaction_query_results] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var interactionType = null;
        var interactionId = null;
        var processId = null;
        var folder = null;
        var campaign = null;
        var transactionType = null;
        var transactionId = null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (req.body) {
                interactionType = req.body.interactionType || null;
                interactionId = req.body.interactionId || null;
                folder = req.body.folder || null;
                campaign = req.body.campaign || null;
                transactionId = req.body.transactionId || null;
                transactionType = req.body.transactionType || null;
            }
            else {
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            }
            logger.info('[InteractionService::get_interaction_query_results] Searching inetraction instances');
            return mssql_service_1.MSSql.GetInteractionQueryResults(vcc, transactionType, transactionId, interactionType, interactionId, campaign, folder);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::get_interaction_query_results] Received results');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::get_interaction_query_results] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/discard_interaction', function (req, res, next) {
        logger.info('[InteractionService::discard_interaction] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userRole = res.locals.user.role;
        var userId = res.locals.userId;
        var evaluationToDiscard = (req.body.evaluation || {});
        var multiple = req.body.multiple || false;
        var saveResult = null;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return role_service_1.GetRoleFunctions(vcc, userId, userRole);
        })
            .then(function (result) {
            if (result && result.data && result.data.hasAdminFunctions)
                throw 'SERVER_ERROR_USER_HAS_NOT_PERMISSION';
            return redis_service_1.Redis.GetEvaluationStatus(vcc, evaluationToDiscard);
        })
            .then(function (result) {
            if (result)
                throw 'SERVER_ERROR_EVALUATION_IS_LOCKED';
            return mssql_service_1.MSSql.SaveEvaluationStatus(vcc, evaluationToDiscard.evaluationPlanId, evaluationToDiscard.interactionId, evaluationToDiscard.auditorId, false, multiple, userId);
        })
            .then(function (result) {
            if (result && result.status === false)
                throw result.error;
            saveResult = result;
            return mssql_service_1.MSSql.SaveEvaluationPlanCompleted(vcc, evaluationToDiscard.evaluationPlanId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::discard_interaction] The evaluation update has been finished.');
            res.json({ status: true, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::discard_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/recover_interaction', function (req, res, next) {
        logger.info('[InteractionService::recover_interaction] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userRole = res.locals.user.role;
        var userId = res.locals.userId;
        var evaluationToDiscard = (req.body.evaluation || {});
        var multiple = req.body.multiple || false;
        var saveResult = null;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return role_service_1.GetRoleFunctions(vcc, userId, userRole);
        })
            .then(function (result) {
            if (result && result.data && result.data.hasAdminFunctions) { }
            else
                throw 'SERVER_ERROR_USER_HAS_NOT_PERMISSION';
            return mssql_service_1.MSSql.SaveEvaluationStatus(vcc, evaluationToDiscard.evaluationPlanId, evaluationToDiscard.interactionId, evaluationToDiscard.auditorId, true, multiple, userId);
        })
            .then(function (result) {
            if (result && result.status === false)
                throw result.error;
            saveResult = result;
            return mssql_service_1.MSSql.SaveEvaluationPlanCompleted(vcc, evaluationToDiscard.evaluationPlanId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::recover_interaction] The evaluation update has been finished.');
            res.json({ status: true, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::recover_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
}
exports.InteractionService = InteractionService;
function fillInteractionContentURL(i, vcc) {
    if (i.interactionType === 'CALL') {
        i.interactionPartGroups.forEach(function (group) {
            group.parts.forEach(function (p) {
                p.partURL = config_1.QUALITY_AUDIO_BASE_URL + i.id + '/' + p.section + '/' + p.part;
            });
        });
    }
}
function fillInteractionTags(instance, interactions) {
    var p = [];
    interactions.forEach(function (i) {
        if (i.interactionType === 'CALL')
            p.push(mssql_service_1.MSSql.GetAllTagsInteraction(instance, i.id)
                .then(function (tags) {
                i.tags = tags || [];
                return i;
            }).catch(function (err) {
                var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                logger.error('Error obteniendo info de los tags: ' + errorMsg);
                throw err;
            }));
        else
            p.push(Promise.resolve(i));
    });
    return Promise.all(p).then(function (values) {
        return [].concat.apply([], values);
    });
}
function GetRemoteInteractionFile(filePath, folder) {
    var genericResult = {};
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return GetRemoteInteractionFileContent(filePath);
    }).then(function (gr) {
        genericResult = gr;
        genericResult.result = folder;
        return Promise.resolve(genericResult);
    }).catch(function (err) {
        genericResult.status = false;
        genericResult.description = 'ERROR';
        genericResult.data = err.toString();
        return Promise.resolve(genericResult);
    });
}
exports.GetRemoteInteractionFile = GetRemoteInteractionFile;
function GetRemoteInteractionFileContent(filePath) {
    var genericResult = {};
    return new Promise(function (resolve, reject) {
        request.get(filePath)
            .on('response', function (response) {
            if (response && response.headers && response.statusCode) {
                if (response.statusCode === 200) {
                    genericResult.status = true;
                    genericResult.description = filePath.split('/').pop();
                    ;
                    genericResult.data = '';
                    response.on('data', function (chunk) {
                        genericResult.data += chunk;
                    });
                    response.on('end', function () {
                        if (filePath.includes('.eml')) {
                            var EmlParser = require('eml-parser');
                            return new EmlParser(genericResult.data)
                                .parseEml()
                                .then(function (data) {
                                console.log(data);
                                genericResult.data = data.html ?
                                    data.html
                                    :
                                        data.text ?
                                            data.text
                                            :
                                                'MESSAGE_CONTENT_NOT_FOUND';
                                if (data.attachments && data.attachments.length > 0) {
                                    genericResult.extraContent = [];
                                    data.attachments.map(function (da) {
                                        var cid = 'cid:';
                                        var buff = new Buffer(da.content);
                                        var base64 = buff.toString('base64');
                                        var format = 'data:' + da.contentType.split(';')[0] + ';base64,';
                                        if (da.cid) {
                                            cid += da.cid.replace('<', '').replace('>', '');
                                        }
                                        else {
                                            cid += 'vmime';
                                        }
                                        if (genericResult.data.includes(cid)) {
                                            var attachmentSrc = format + base64;
                                            if (cid === 'cid:vmime') {
                                                var imgSrcPattern = /src\s*=\s*"cid:vmime(.+?)"/;
                                                genericResult.data = genericResult.data.replace(imgSrcPattern, 'src="' + attachmentSrc + '"');
                                            }
                                            else {
                                                genericResult.data = genericResult.data.replace(cid, attachmentSrc);
                                            }
                                        }
                                        if (da.type && da.type === 'attachment' && !da.related) {
                                            var aFile = {};
                                            aFile.id = da.id;
                                            aFile.contentType = da.contentType.split(';')[0];
                                            aFile.base64 = base64;
                                            if (da.filename) {
                                                aFile.name = da.filename;
                                            }
                                            else {
                                                var counter = genericResult.extraContent.filter(function (ec) { return ec.name.includes('untitled'); }).length;
                                                counter = counter + 1;
                                                aFile.name = 'untitled_' + counter + '.' + aFile.contentType.split('/')[1];
                                            }
                                            genericResult.extraContent.push(aFile);
                                        }
                                    });
                                }
                                return resolve(genericResult);
                            });
                        }
                        else {
                            return resolve(genericResult);
                        }
                    });
                }
                else {
                    genericResult.status = false;
                    genericResult.result = 'GET_TEXT_REQUEST_FAILED';
                    return resolve(genericResult);
                }
            }
            else {
                logger.error('[InteractionService::GetRemoteInteractionFileContent] Getting errorMessage: NO_RESPONSE_HEADERS');
                genericResult.status = false;
                genericResult.error = 'NO_RESPONSE_HEADERS';
                return resolve(genericResult);
            }
        })
            .on('error', function (err) {
            if (err !== undefined && err !== null) {
                var errorMessage = typeof err === 'string' ? err : err.message;
                logger.error('[InteractionService::GetRemoteInteractionFileContent] Cannot download audio from: ', filePath, 'getting errorMessage:', errorMessage);
                genericResult.status = false;
                genericResult.error = err.message;
                return resolve(genericResult);
            }
        });
    });
}
exports.GetRemoteInteractionFileContent = GetRemoteInteractionFileContent;
function GetLocalInteractionFileContent(interaction, fileName) {
    var filePath = '';
    var resultReadFile = '';
    var genericResult = {};
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        if (interaction.interactionType !== 'CALL') {
            filePath += config_1.TRANSCRIPTION_FOLDER;
        }
        else {
            filePath += config_1.SEMANCTIC_ANALYSIS_FOLDER;
        }
        filePath += '/' + interaction.vcc + '/' + fileName;
        return file_manager_1.ExistInFileSystem(filePath);
    }).then(function (result) {
        if (result === true) {
            return file_manager_1.ReadFileContent(filePath, 'utf-8');
        }
        else {
            throw 'SERVER_ERROR_LOCAL_FILE_NOT_FOUND';
        }
    }).then(function (result) {
        if (result && result.length > 0) {
            resultReadFile = result;
            if (fileName.includes('.eml')) {
                var EmlParser = require('eml-parser');
                return new EmlParser(genericResult.data)
                    .parseEml()
                    .then(function (data) {
                    console.log(data);
                    genericResult.data = data.html ?
                        data.html
                        :
                            data.text ?
                                data.text
                                :
                                    'MESSAGE_CONTENT_NOT_FOUND';
                    if (data.attachments && data.attachments.length > 0) {
                        genericResult.extraContent = [];
                        data.attachments.map(function (da) {
                            var cid = 'cid:';
                            var buff = new Buffer(da.content);
                            var base64 = buff.toString('base64');
                            var format = 'data:' + da.contentType.split(';')[0] + ';base64,';
                            if (da.cid) {
                                cid += da.cid.replace('<', '').replace('>', '');
                            }
                            else {
                                cid += 'vmime';
                            }
                            if (genericResult.data.includes(cid)) {
                                var attachmentSrc = format + base64;
                                if (cid === 'cid:vmime') {
                                    var imgSrcPattern = /src\s*=\s*"cid:vmime(.+?)"/;
                                    genericResult.data = genericResult.data.replace(imgSrcPattern, 'src="' + attachmentSrc + '"');
                                }
                                else {
                                    genericResult.data = genericResult.data.replace(cid, attachmentSrc);
                                }
                            }
                            if (da.type && da.type === 'attachment' && !da.related) {
                                var aFile = {};
                                aFile.id = da.id;
                                aFile.contentType = da.contentType.split(';')[0];
                                aFile.base64 = base64;
                                if (da.filename) {
                                    aFile.name = da.filename;
                                }
                                else {
                                    var counter = genericResult.extraContent.filter(function (ec) { return ec.name.includes('untitled'); }).length;
                                    counter = counter + 1;
                                    aFile.name = 'untitled_' + counter + '.' + aFile.contentType.split('/')[1];
                                }
                                genericResult.extraContent.push(aFile);
                            }
                        });
                    }
                    return bluebird_1.resolve(genericResult);
                });
            }
            else {
                return resultReadFile;
            }
        }
        else {
            throw 'SERVER_ERROR_CANNOT_READ_LOCAL_FILE';
        }
    }).then(function (result) {
        if (resultReadFile) {
            if (resultReadFile && resultReadFile.length > 0) {
                genericResult.status = true;
                genericResult.description = fileName;
                genericResult.data = resultReadFile;
                return Promise.resolve(genericResult);
            }
            else {
                throw 'SERVER_ERROR_FILE_IS_EMPTY';
            }
        }
        else {
            throw 'SERVER_ERROR_FILE_IS_EMPTY';
        }
    }).catch(function (err) {
        genericResult.status = false;
        genericResult.description = 'ERROR';
        genericResult.data = err.toString();
        return Promise.resolve(genericResult);
    });
}
exports.GetLocalInteractionFileContent = GetLocalInteractionFileContent;
function GetEvaluationPlanInteraction(vcc, interactionType, environmentType, evaluationPlanId, interactionId) {
    var interaction = {};
    var interactionParts = [];
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        if (interactionType === 'CALL' && environmentType && environmentType === 'allegro') {
            logger.info('[InteractionService::GetEvaluationPlanInteraction] Requesting to HistoricalData');
            return ExternalDB.interactions.getInteractionByTypeAndId(vcc, interactionType, interactionId, true);
        }
        else {
            return mssql_service_1.MSSql.GetEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId);
        }
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_INTERACTION_NOT_FOUND';
        interaction = result;
        if (interactionType === 'CALL' && interaction && interaction.needI6Update) {
            return UpdateI6Interaction({ vcc: vcc, id: interactionId });
        }
        else {
            return Promise.resolve(null);
        }
    })
        .then(function (_) {
        if (interactionType === 'CALL' && environmentType && environmentType === 'i6') {
            return mssql_service_1.MSSql.GetI6InteractionAudios(vcc, interactionId);
        }
        else {
            return Promise.resolve(null);
        }
    })
        .then(function (result) {
        interactionParts = result;
        if (interactionType === 'CALL' && environmentType && environmentType === 'i6') {
            if (interactionParts && !interactionParts.length) {
                return http_request_occ_1.ApiI6GetInteractionData(vcc, interactionId);
            }
        }
        return Promise.resolve(result);
    })
        .then(function (result) {
        if (interactionType === 'CALL' && environmentType && environmentType === 'i6' && interactionParts.length === 0) {
            var i6Data = null;
            var i6Audios_1 = [];
            i6Data = result;
            i6Data.body.interactionData.records = i6Data.body.interactionData.records.filter(function (r) { return r.Format !== 'mp4'; });
            i6Data.body.interactionData.records = arrays_shared_1.SortObjectArrayByValue(i6Data.body.interactionData.records, 'Part', 'A');
            i6Data.body.interactionData.records.map(function (r) {
                i6Audios_1.push({
                    id: interactionId,
                    section: r.Section,
                    part: r.Part,
                    agentId: r.agent.includes('@') ? r.agent.split('@')[0] : r.agent,
                    format: r.Format
                });
            });
            return Promise.resolve(i6Audios_1);
        }
        return Promise.resolve(result);
    })
        .then(function (result) {
        if (interactionType === 'CALL' && environmentType && environmentType === 'i6') {
            if (interactionParts.length === 0)
                interactionParts = result;
            interaction.interactionPartGroups = [];
            var uniqueAgentItems = arrays_shared_1.GetUniqueFromObjectArray(interactionParts, 'agentId');
            uniqueAgentItems.map(function (ua) {
                var item = {};
                item.agentId = ua.agentId;
                item.parts = interactionParts.filter(function (ip) { return ip.agentId === item.agentId; });
                interaction.interactionPartGroups.push(item);
            });
        }
        return Promise.resolve(true);
    })
        .then(function (_) {
        return Promise.resolve(interaction);
    })
        .catch(function (err) {
        logger.info('[InteractionService::GetEvaluationPlanInteraction] Error: ', err);
        return Promise.reject(err);
    });
}
function GetInteractionSearchFromRequest(evaluationFormatId, filtersObject, withSectionPart) {
    var params = new interaction_model_1.InteractionSearch;
    params.evaluationFormatId = evaluationFormatId || null;
    params.withSectionPart = withSectionPart;
    params.environmentId = filtersObject.environmentId || null;
    params.environmentType = filtersObject.environmentType || null;
    params.types = filtersObject.types || [];
    params.dateIni = filtersObject.dateIni ? moment(filtersObject.dateIni).toDate() : null;
    params.dateFin = filtersObject.dateFin ? moment(filtersObject.dateFin).toDate() : null;
    params.hourIni = filtersObject.hourIni || null;
    params.hourFin = filtersObject.hourFin || null;
    params.minDuration = filtersObject.minDuration || null;
    params.maxDuration = filtersObject.maxDuration || null;
    params.campaigns = filtersObject.campaigns || [];
    params.agents = filtersObject.agents || [];
    params.results = filtersObject.results || [];
    params.origins = filtersObject.origins || [];
    params.multimediaFormats = JSON.parse(JSON.stringify(config_1.DEFAULT_INTERACTION_FORMAT));
    params.evaluated = filtersObject.evaluated || [];
    if (params.evaluationFormatId) {
        params.evaluationTypes = filtersObject.evaluationTypes || [];
        params.tags = filtersObject.tags || [];
        params.hasVideoOptions = filtersObject.hasVideoOptions || [];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
    }
    params.interactionIds = filtersObject.interactionIds ? filtersObject.interactionIds : [];
    return params;
}
exports.GetInteractionSearchFromRequest = GetInteractionSearchFromRequest;
function GetProcessFilterFromInteractionSearch(processId, interactionSearch, userId) {
    var processFilterArray = [];
    var interactionSearchKeys = Object.keys(interactionSearch);
    var uniqueItemArray = ['dateIni', 'dateFin', 'hourIni', 'hourFin', 'minDuration', 'maxDuration', 'interactionType'];
    var multipleItemArray = ['campaigns', 'types', 'origins', 'results', 'interactionIds', 'agents', 'evaluated'];
    interactionSearchKeys.map(function (isk) {
        var keyName = '';
        switch (isk) {
            case 'dateIni':
                keyName = 'startDate';
                break;
            case 'dateFin':
                keyName = 'endDate';
                break;
            case 'hourIni':
                keyName = 'startTime';
                break;
            case 'hourFin':
                keyName = 'endTime';
                break;
            default:
                keyName = isk;
                break;
        }
        if (uniqueItemArray.includes(isk)) {
            var uniqueValue = '';
            if (isk === 'agents')
                uniqueValue = interactionSearch[isk].join('-');
            else if (isk === 'dateIni')
                uniqueValue = interactionSearch[isk].toISOString();
            else if (isk === 'dateFin')
                uniqueValue = interactionSearch[isk].toISOString();
            else
                uniqueValue = interactionSearch[isk];
            if (uniqueValue !== null && uniqueValue !== '') {
                processFilterArray.push(new process_model_1.ProcessFilter(processId, keyName, uniqueValue, moment().utc().toDate(), userId, moment().utc().toDate(), userId));
            }
        }
        else if (multipleItemArray.includes(isk)) {
            var multipleValue = [];
            if (isk === 'origins') {
                if (interactionSearch[isk].length === 1)
                    multipleValue = interactionSearch[isk];
            }
            else
                multipleValue = interactionSearch[isk];
            multipleValue.map(function (mv) {
                if (mv !== null && mv !== '') {
                    processFilterArray.push(new process_model_1.ProcessFilter(processId, keyName, mv, moment().utc().toDate(), userId, moment().utc().toDate(), userId));
                }
            });
        }
    });
    return processFilterArray;
}
exports.GetProcessFilterFromInteractionSearch = GetProcessFilterFromInteractionSearch;
function GetChannelNewInteractionSummaryPromises(vcc, vccType, isSummary, resultType, filters, csvData) {
    if (filters.types
        &&
            ((filters.types.includes('CALL') && filters.types.length > 1)
                ||
                    (!filters.types.includes('CALL') && filters.types.length > 0))) {
        if (!integration_config_1.IP_SQL_I6 || integration_config_1.IP_SQL_I6 === '') {
            filters.types = ['CALL'];
        }
    }
    var promiseArray = [];
    var results = filters.results || [];
    var result_numbers = [];
    var result_texts = [];
    if (results.length > 0) {
        results.map(function (r) {
            if (typeof r === 'string') {
                result_texts.push(r);
                result_numbers.push(r);
            }
            else {
                if (r.code) {
                    var rcf = JSON.parse(JSON.stringify(r));
                    result_numbers.push(rcf.campaignId + '_' + rcf.code);
                }
                else {
                    var amr = JSON.parse(JSON.stringify(r));
                    var resultText = '';
                    if (amr.description)
                        resultText = '/' + amr.description;
                    if (amr.description2)
                        resultText += '/' + amr.description2;
                    if (amr.description3)
                        resultText += '/' + amr.description3;
                    if (amr.description4)
                        resultText += '/' + amr.description4;
                    result_texts.push(resultText);
                }
            }
        });
        if (result_numbers.length === 0)
            result_numbers.push('NULL');
        if (result_texts.length === 0)
            result_texts.push('NULL');
    }
    if (!filters.multimediaFormats)
        filters.multimediaFormats = null;
    if (filters.environmentType && (filters.environmentType !== 'allegro' && filters.environmentType !== 'i6'))
        vccType = 'OTHER';
    var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
    switch (vccType) {
        case 'ALLEGRO':
            if (filters.types.includes('CALL'))
                promiseArray.push(mssql_service_1.MSSql.GetAllegroInteractions(integration_config_1.IP_BD, vcc, filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_numbers, flagAllegro, filters.origins.toString(), filters.minDuration, filters.maxDuration, filters.multimediaFormats ? filters.multimediaFormats.toString() : null, filters.interactionIds.toString(), filters.evaluated.toString(), resultType));
            break;
        case 'I6':
            if (filters.types.includes('APPSTORE'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'APPSTORE', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('CALL'))
                promiseArray.push(mssql_service_1.MSSql.GetI6CallInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'CALL', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.minDuration, filters.maxDuration, filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('CHAT'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'CHAT', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('FACEBOOK'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'FACEBOOK', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('GOOGLEPLAY'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'GOOGLEPLAY', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('INSTAGRAM'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'INSTAGRAM', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('MAIL'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'MAIL', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('TWITTER'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'TWITTER', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('WEBCONTACT'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'WEBCONTACT', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('WHATSAPP'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'WHATSAPP', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('YOUTUBE'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'YOUTUBE', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            break;
        case 'OCC':
            if (filters.types.includes('CALL'))
                promiseArray.push(mssql_service_1.MSSql.GetAllegroInteractions(integration_config_1.IP_BD, vcc, filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_numbers, flagAllegro, filters.origins.toString(), filters.minDuration, filters.maxDuration, filters.multimediaFormats ? filters.multimediaFormats.toString() : null, filters.interactionIds.toString(), filters.evaluated.toString(), resultType));
            if (filters.types.includes('APPSTORE'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'APPSTORE', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('CHAT'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'CHAT', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('FACEBOOK'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'FACEBOOK', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('GOOGLEPLAY'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'GOOGLEPLAY', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('INSTAGRAM'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'INSTAGRAM', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('MAIL'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'MAIL', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('TWITTER'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'TWITTER', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('WEBCONTACT'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'WEBCONTACT', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('WHATSAPP'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'WHATSAPP', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            if (filters.types.includes('YOUTUBE'))
                promiseArray.push(mssql_service_1.MSSql.GetI6TextInteractions(integration_config_1.IP_SQL_I6, vcc, flagAllegro, 'YOUTUBE', filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_texts, filters.origins.toString(), filters.interactionIds.toString(), filters.evaluated.toString(), isSummary, resultType));
            break;
        case 'OTHER':
            promiseArray.push(mssql_service_1.MSSql.BulkCSVSummaryInteractions(csvData, vcc, filters.dateIni, filters.dateFin, filters.campaigns.toString(), filters.agents.toString(), filters.origins.toString(), filters.hourIni, filters.hourFin, filters.minDuration, filters.maxDuration, result_numbers, isSummary, filters.interactionIds.toString()));
            break;
    }
    return promiseArray;
}
exports.GetChannelNewInteractionSummaryPromises = GetChannelNewInteractionSummaryPromises;
function GetChannelOldInteractionSummaryPromises(vcc, vccType, modelSecondsPerRequest, modelCharsPerPhrase, filters, csvData) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var promiseArray = [];
        promiseArray = GetChannelNewInteractionSummaryPromises(vcc, vccType, false, 'DETAILED', filters, csvData);
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var promiseArray = [];
        result.map(function (r) {
            promiseArray.push(mssql_service_1.MSSql.BulkBillingInteractions(vcc, modelSecondsPerRequest, modelCharsPerPhrase, r.interactionType, r.interactions));
        });
        return Promise.all(promiseArray);
    })
        .catch(function (err) {
        return Promise.resolve([]);
    });
}
exports.GetChannelOldInteractionSummaryPromises = GetChannelOldInteractionSummaryPromises;
function UpdateI6Interaction(interaction) {
    var i6Data = null;
    var i6Audios = [];
    var bdAudios = [];
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return http_request_occ_1.ApiI6GetInteractionData(interaction.vcc, interaction.id);
    })
        .then(function (result) {
        if (!result || (result && !result.status))
            throw 'SERVER_ERROR_I6_INTERACTION_DATA_NOT_FOUND';
        i6Data = result;
        return mssql_service_1.MSSql.GetI6InteractionAudios(interaction.vcc, interaction.id);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_BD_INTERACTION_DATA_NOT_FOUND';
        var promiseArray = [];
        bdAudios = result;
        i6Data.body.interactionData.records = i6Data.body.interactionData.records.filter(function (r) { return r.Format !== 'mp4'; });
        i6Data.body.interactionData.records = arrays_shared_1.SortObjectArrayByValue(i6Data.body.interactionData.records, 'Part', 'A');
        for (var i = 0; i < bdAudios.length; i++) {
            i6Audios.push(i6Data.body.interactionData.records.shift());
        }
        var classifiedData = arrays_shared_1.GetCompareItemsFromObjectArray(bdAudios, i6Audios, 'part', 'Part');
        classifiedData.old.map(function (o) {
            var i6Audio = i6Audios.find(function (r) { return r.Part === o.part; });
            promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAudio(interaction.vcc, interaction.id, o.section, o.part, i6Audio.Section || i6Audio.section || 0, i6Audio.Part, i6Audio.agent && i6Audio.agent.includes('@') ? i6Audio.agent.split('@')[0] : null, i6Audio.Format || i6Audio.format));
            if (interaction.section
                &&
                    interaction.section === o.section
                &&
                    interaction.part
                &&
                    interaction.part === o.part) {
                interaction.agent = i6Audio.agent && i6Audio.agent.includes('@') ? i6Audio.agent.split('@')[0] : null;
                interaction.format = i6Audio.Format || i6Audio.format;
            }
        });
        classifiedData.new.map(function (n) {
            var currentAudio = classifiedData.removed.shift();
            promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAudio(interaction.vcc, interaction.id, currentAudio.section, currentAudio.part, n.Section || n.section || 0, n.Part, n.agent && n.agent.includes('@') ? n.agent.split('@')[0] : null, n.Format || n.format));
            if (interaction.section
                &&
                    interaction.section === currentAudio.section
                &&
                    interaction.part
                &&
                    interaction.part === currentAudio.part) {
                interaction.agent = n.agent && n.agent.includes('@') ? n.agent.split('@')[0] : null;
                interaction.format = n.Format || n.format;
            }
        });
        classifiedData.removed.map(function (r) {
            var bdAudio = bdAudios.find(function (a) { return a.part === r.Part; });
            if (bdAudio) {
                promiseArray.push(mssql_service_1.MSSql.DeleteI6InteractionAudio(interaction.vcc, interaction.id, bdAudio.section, bdAudio.part));
            }
        });
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        return Promise.resolve(interaction);
    })
        .catch(function (err) {
        return Promise.reject(err);
    });
}
exports.UpdateI6Interaction = UpdateI6Interaction;
