"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReportService = void 0;
var log4js = require("log4js");
var moment = require("moment");
var report_implementation_1 = require("../implementation/report.implementation");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function ReportService(app) {
    app.get('/api/report/get_reports/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[ReportService::get_reports] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = res.locals.userRole;
        var userId = res.locals.userId;
        if (userId === 'admin' && vcc === 'system') {
            role.id = 'provisioning';
        }
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (role.id == 'business_administrator' || role.id == 'provisioning') {
                return mssql_service_1.MSSql.GetReports(vcc, search_criteria, search_expression, role.id);
            }
            else {
                return mssql_service_1.MSSql.GetReports(vcc, search_criteria, search_expression, null);
            }
        })
            .then(function (resultReports) {
            if (!resultReports)
                throw 'REPORT_SERVER_ERROR_CANNOT_EXECUTE';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultReports });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/report/get_report/:report_id', function (req, res, next) {
        logger.info('[ReportService::get_report] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = res.locals.userRole;
        var reportId = req.params.report_id;
        var userId = res.locals.userId;
        if (userId === 'admin') {
            role.id = 'business_administrator';
        }
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!reportId)
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            if (role.id == 'business_administrator') {
                return mssql_service_1.MSSql.GetReport(vcc, reportId, role.id);
            }
            else {
                return mssql_service_1.MSSql.GetReport(vcc, reportId, null);
            }
        }).then(function (report) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: report });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/create_report', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newReport = (req.body || {});
        newReport.createdDate = nowUtc;
        newReport.createdByUserId = userId;
        newReport.lastModifiedDate = nowUtc;
        newReport.lastModifiedByUserId = userId;
        newReport.startDate = (newReport.startDate ? new Date(newReport.startDate) : null);
        newReport.endDate = (newReport.endDate ? new Date(newReport.endDate) : null);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetReport(vcc, newReport.id);
        }).then(function (resultReport) {
            if (resultReport)
                throw 'REPORT_SERVER_ERROR_EXISTS';
            return mssql_service_1.MSSql.SaveReport(vcc, newReport.id, newReport.description, newReport.type, newReport.procedure, newReport.startDate, newReport.endDate, newReport.frecuency, newReport.hourSpace, newReport.exactTime, newReport.weekday, newReport.dayMonth, newReport.summary, newReport.filter, newReport.email, newReport.createdDate, newReport.createdByUserId, newReport.lastModifiedDate, newReport.lastModifiedByUserId);
        }).then(function (resultSaveReport) {
            if (!resultSaveReport)
                throw 'REPORT_SERVER_ERROR_ON_SAVE';
            return mssql_service_1.MSSql.SaveSinglePermission(vcc, 'REPORT', newReport.id);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('Reporte \'' + newReport.id + '\' creado correctamente');
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'REPORT_SERVER_ERROR_ON_SAVE'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/save_report', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var role = res.locals.userRole;
        var nowUtc = moment().utc().toDate();
        var newReport = (req.body || {});
        newReport.createdDate = null;
        newReport.createdByUserId = null;
        newReport.lastModifiedDate = nowUtc;
        newReport.lastModifiedByUserId = userId;
        newReport.startDate = (newReport.startDate ? new Date(newReport.startDate) : null);
        newReport.endDate = (newReport.endDate ? new Date(newReport.endDate) : null);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(function (sp) { return sp.type === 'REPORT'; }).map(function (sp) { return sp.element; }) :
                []);
            if (!allowedReports.includes(newReport.id)) {
                res.sendStatus(403);
                throw 'REPORT_SERVER_ERROR_ACCESS_DENIED';
            }
            return mssql_service_1.MSSql.SaveReport(vcc, newReport.id, newReport.description, newReport.type, newReport.procedure, newReport.startDate, newReport.endDate, newReport.frecuency, newReport.hourSpace, newReport.exactTime, newReport.weekday, newReport.dayMonth, newReport.summary, newReport.filter, newReport.email, newReport.createdDate, newReport.createdByUserId, newReport.lastModifiedDate, newReport.lastModifiedByUserId);
        }).then(function (resultSaveReport) {
            if (!resultSaveReport)
                throw 'REPORT_SERVER_ERROR_ON_SAVE';
            return mssql_service_1.MSSql.SaveSinglePermission(vcc, 'REPORT', newReport.id);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('Reporte \'' + newReport.id + '\' actualizado correctamente');
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'REPORT_SERVER_ERROR_ON_SAVE'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/delete_report', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = res.locals.userRole;
        var report = req.body.report || {};
        var reportId = report.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!report)
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            var allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(function (sp) { return sp.type === 'REPORT'; }).map(function (sp) { return sp.element; }) :
                []);
            if (!allowedReports.includes(reportId)) {
                res.sendStatus(403);
                throw 'REPORT_SERVER_ERROR_ACCESS_DENIED';
            }
            return mssql_service_1.MSSql.DeleteReport(vcc, reportId);
        }).then(function (resultDeleteReport) {
            if (!resultDeleteReport)
                throw 'REPORT_SERVER_ERROR_ON_SAVE';
            return mssql_service_1.MSSql.DeleteSinglePermission(vcc, 'REPORT', reportId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('Reporte \'' + reportId + '\' eliminado correctamente');
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'REPORT_SERVER_ERROR_ON_DELETE'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/delete_reports', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = res.locals.userRole;
        var reports = req.body.reports || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!reports)
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            var allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(function (sp) { return sp.type === 'REPORT'; }).map(function (sp) { return sp.element; }) :
                []);
            var forbiddenReportId;
            reports.forEach(function (report) {
                if (!forbiddenReportId && !allowedReports.includes(report.id)) {
                    forbiddenReportId = report.id;
                }
            });
            if (forbiddenReportId) {
                res.sendStatus(403);
                throw 'REPORT_SERVER_ERROR_ACCESS_DENIED';
            }
            var promiseArray = [];
            reports.forEach(function (report) {
                promiseArray.push(mssql_service_1.MSSql.DeleteReport(vcc, report.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteReports) {
            var result;
            if (resultDeleteReports.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Reportes eliminados correctamente');
            }
            var promiseArray = [];
            reports.forEach(function (report) {
                promiseArray.push(mssql_service_1.MSSql.DeleteSinglePermission(vcc, 'REPORT', report.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteSinglePermissions) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteSinglePermissions, description: (resultDeleteSinglePermissions ? 'SERVER_SUCCESS_MESSAGE' : 'REPORT_SERVER_ERROR_ON_DELETE'), data: resultDeleteSinglePermissions });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/report/get_report_procedures', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetReportProcedures();
        }).then(function (response) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: response });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/report/get_report_procedure_parameters/:procedure', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var role = res.locals.userRole;
        var userId = res.locals.userId;
        var procedure = req.params.procedure;
        var fromBilling = false;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (role.id === 'business_administrator' || userId === 'admin') {
                var report = new report_implementation_1.ReportImplementation();
                fromBilling = true;
                return report.GetReportProceduraParameters(procedure);
            }
            else {
                return mssql_service_1.MSSql.GetReportProcedureParameters(procedure);
            }
        }).then(function (response) {
            if (res.headersSent)
                return;
            var procedureParameter = [];
            if (response) {
                response.forEach(function (parameter) {
                    if (userId === 'admin') {
                        if (parameter.parameter !== 'PlatformHash' && parameter.parameter !== '@VCC' && parameter.parameter !== '@UserId' && parameter.parameter !== '@ClientOffset' && !parameter.isOutput) {
                            if (!fromBilling) {
                                if (parameter.type === 'datetime') {
                                    parameter.type = 'date';
                                }
                                else {
                                    parameter.type = 'text';
                                }
                            }
                            parameter.parameter = parameter.parameter.replace('@', '');
                            procedureParameter.push(parameter);
                        }
                    }
                    else {
                        if (parameter.parameter !== 'SubAccountId' && parameter.parameter !== 'AccountId' && parameter.parameter !== 'InstallationId' && parameter.parameter !== 'PlatformHash' && parameter.parameter !== 'VCC' && parameter.parameter !== '@VCC' && parameter.parameter !== '@UserId' && parameter.parameter !== '@ClientOffset' && !parameter.isOutput) {
                            if (fromBilling) {
                                if (parameter.type === 'datetime') {
                                    parameter.type = 'date';
                                }
                                else {
                                    parameter.type = 'text';
                                }
                            }
                            parameter.parameter = parameter.parameter.replace('@', '');
                            procedureParameter.push(parameter);
                        }
                    }
                });
            }
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: procedureParameter });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/get_filters_options', function (req, res) {
        var filters = req.body || [];
        var VCC = res.locals.vcc;
        var UserId = res.locals.userId;
        promises_shared_1.DummyPromise().then(function (_) {
            var promises = [];
            for (var _i = 0, filters_1 = filters; _i < filters_1.length; _i++) {
                var f = filters_1[_i];
                var parameters = [];
                if (f._arguments) {
                    var _arguments = f._arguments.split(",");
                    for (var _a = 0, _arguments_1 = _arguments; _a < _arguments_1.length; _a++) {
                        var a = _arguments_1[_a];
                        var value = '';
                        eval("value=" + a);
                        parameters.push({
                            column: a,
                            value: value
                        });
                    }
                }
                promises.push(mssql_service_1.MSSql.ExecuteReportProcedure(f.procedure, parameters));
            }
            return Promise.all(promises);
        }).then(function (results) {
            var response = {};
            var i = 0;
            for (var _i = 0, filters_2 = filters; _i < filters_2.length; _i++) {
                var f = filters_2[_i];
                response[f.column] = results[i++];
            }
            res.send({
                status: true, description: 'Ok', data: response
            });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/execute_report', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var role = res.locals.userRole;
        var reportExecution = req.body.reportExecution || {};
        var procedure = reportExecution.procedure;
        var filters = reportExecution.filters;
        var clientOffset = reportExecution.clientOffset;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!filters && !procedure)
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            if (role.id === 'business_administrator' || userId === 'admin') {
                var report = new report_implementation_1.ReportImplementation();
                return report.GetReportProceduraParameters(procedure);
            }
            else {
                return mssql_service_1.MSSql.GetReportProcedureParameters(procedure);
            }
        })
            .then(function (resultParameters) {
            var procedureParameters = [];
            resultParameters.forEach(function (parameter) {
                var column = parameter.parameter.replace('@', '');
                var colFilter = filters.find(function (filter) { return filter && filter.column === column; });
                var value = (colFilter && colFilter.value ? colFilter.value : null);
                if (value && (column === 'EndDate' || column === 'StartDate')) {
                    var momentDate = moment.parseZone(value).isValid() && moment.parseZone(value) || null;
                    value = (momentDate) ? momentDate.utc().add(clientOffset, 'minutes').toDate() : null;
                }
                if (!value && column === 'VCC')
                    value = vcc;
                else if (!value && column === 'UserId')
                    value = userId;
                else if (!value && column === 'ClientOffset')
                    value = clientOffset;
                if (column !== 'RETURN_VALUE')
                    procedureParameters.push({
                        column: column,
                        value: value
                    });
            });
            if (role.id === 'business_administrator' || userId === 'admin') {
                var report = new report_implementation_1.ReportImplementation();
                return report.ReportExecution(procedureParameters, procedure, clientOffset);
            }
            else {
                return mssql_service_1.MSSql.ExecuteReportProcedure(procedure, procedureParameters);
            }
        }).then(function (responseReportData) {
            if (res.headersSent)
                return;
            var columns = [];
            var transformColumns = ['DateTime'];
            if (responseReportData) {
                for (var key in responseReportData.columns) {
                    var c = responseReportData.columns[key];
                    var t = c.type ? c.type.name || c.type.declaration : '';
                    if (transformColumns.includes(t)) {
                        columns.push({ name: c.name, type: t });
                    }
                }
            }
            if (columns.length > 0) {
                for (var _i = 0, responseReportData_1 = responseReportData; _i < responseReportData_1.length; _i++) {
                    var row = responseReportData_1[_i];
                    for (var _a = 0, columns_1 = columns; _a < columns_1.length; _a++) {
                        var c = columns_1[_a];
                        var value = row[c.name];
                        if (value) {
                            var momentDate = moment.parseZone(value).isValid() && moment.parseZone(value) || null;
                            value = (momentDate) ? momentDate.utc().add(clientOffset, 'minutes').format('YYYY-MM-DD HH:mm:ss') : null;
                            row[c.name] = value;
                        }
                    }
                }
            }
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: responseReportData });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.ReportService = ReportService;
