"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessImplementation = void 0;
var redis_service_1 = require("../db/redis.service");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var billing_implementation_1 = require("../implementation/billing.implementation");
var ability_service_1 = require("../services/ability.service");
var template_service_1 = require("../services/template.service");
var logger = log4js.getLogger('Task');
var ProcessImplementation = (function () {
    function ProcessImplementation() {
    }
    ProcessImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_PROCESS;
        var executeResult = true;
        return redis_service_1.Redis.LRange('Process', 0, ratio - 1)
            .then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            logger.info('[ProcessImplementation::Execute] executionCode:', executionCode, 'ratio:', ratio, 'getting Redis.LRange:', result);
            while (ratio >= executeIndex) {
                promiseArray.push(_this.ProcessExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('Process', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[ProcessImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.ProcessExecute = function (executionCode) {
        var _this = this;
        var currentQueueId = '';
        var vcc = '';
        var processId = '';
        var type = '';
        var typeId = '';
        return redis_service_1.Redis.RPopLPush('Process', 'ProcessExecute')
            .then(function (queueId) {
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[ProcessImplementation::ProcessExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId);
                var item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    type = item[2] ? item[2] : null;
                    typeId = item[3] ? item[3] : null;
                    var promiseArray = [];
                    logger.debug('[ProcessImplementation::ProcessExecute] Creating progress keys for process ' + processId);
                    promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, 0));
                    promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, 0));
                    promiseArray.push(redis_service_1.Redis.SaveErrorProgress(currentQueueId, 0));
                    return Promise.all(promiseArray);
                }
                else {
                    logger.error('[ProcessImplementation::ProcessExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then(function (_) {
            if (vcc && processId && type === 'study' && typeId) {
                return _this.ProcessStudy(vcc, processId, typeId, executionCode, currentQueueId);
            }
            else if (vcc && processId && type === 'evaluationPlan' && typeId) {
                return _this.ProcessEvaluationPlan(vcc, processId, typeId, executionCode, currentQueueId);
            }
            else if (vcc && processId && type === 'ability' && typeId) {
                return _this.ProcessAbility(vcc, processId, typeId, executionCode, currentQueueId);
            }
            else if (vcc && processId && type === 'i6update') {
                return _this.ProcessI6Update(vcc, processId, executionCode, currentQueueId);
            }
            else if (vcc && processId && type === null) {
                return _this.ProcessAudio(vcc, processId, executionCode, currentQueueId);
            }
            return null;
        }).then(function (result) {
            if (result) {
                return redis_service_1.Redis.DeleteQueue('ProcessExecute', currentQueueId);
            }
            else {
                logger.error('[ProcessImplementation::ProcessExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVE');
                return Promise.resolve(false);
            }
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.ProcessI6Update = function (vcc, processId, executionCode, currentQueueId) {
        var _this = this;
        var interactionsToUpdate = [];
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetProcessI6InteractionsToUpdateTask(vcc, processId);
        })
            .then(function (result) {
            if (!result.status)
                throw 'SERVER_ERROR_GET_I6_INTERACTIONS_TASK_FAILED';
            interactionsToUpdate = result && result.data && result.data.length ? result.data : [];
            if (interactionsToUpdate && interactionsToUpdate.length) {
                var promiseArray_1 = [];
                logger.info('[ProcessImplementation::ProcessI6Update] Getting i6 call interactions to update: ' + interactionsToUpdate.length);
                interactionsToUpdate.map(function (itu) {
                    promiseArray_1.push(_this.ProcessI6UpdateExecute(itu, currentQueueId, executionCode));
                });
                return Promise.all(promiseArray_1);
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessI6Update] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'GetProcessI6InteractionsToUpdateTask: 0');
                var processQueue = vcc + ":" + processId;
                logger.error('[ProcessImplementation::ProcessI6Update] Creating process queue');
                return redis_service_1.Redis.CreateQueueL('Process', processQueue);
            }
        })
            .then(function (_) {
            var promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessI6Update] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, interactionsToUpdate.length));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, interactionsToUpdate.length));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessI6Update] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.ProcessAudio = function (vcc, processId, executionCode, currentQueueId) {
        var _this = this;
        var totalAllAudio = 0;
        var totalNewAudio = 0;
        var totalOldAudio = 0;
        var batchTypeName = '';
        var batchTypeId = '';
        var processInteractions = [];
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetProcessInteractionsTask(vcc, processId);
        })
            .then(function (result) {
            if (!result.status)
                throw 'SERVER_ERROR_GET_INTERACTIONS_TASK_FAILED';
            processInteractions = result && result.data && result.data.length ? result.data : [];
            if (processInteractions.filter(function (pi) { return pi.needI6Update; }).length)
                throw 'CANCELLING_PROCESS_BY_UNCOMPLETED_I6_UPDATE';
            return mssql_service_1.MSSql.GetEvaluationPlanByProcessId(vcc, processId);
        })
            .then(function (result) {
            if (result !== null) {
                batchTypeName = 'EVALUATION_PLAN';
                batchTypeId = result.id;
            }
            else {
                batchTypeName = 'TRANSCRIPTION';
                batchTypeId = processId;
            }
            if (processInteractions && processInteractions.length) {
                totalNewAudio = processInteractions.length;
                logger.info('[ProcessImplementation::ProcessAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'GetProcessInteractionsTask:', processInteractions.length);
                var promiseArray_2 = [];
                var billing_1 = new billing_implementation_1.BillingImplementation();
                logger.info('[ProcessImplementation::ProcessAudio] Adding new Transcription to MainFile: ' + processInteractions.length);
                processInteractions.forEach(function (processInteraction) {
                    promiseArray_2.push(_this.ProcessInteractionExecute(processInteraction, currentQueueId, executionCode));
                    billing_1.AddInsertTranscriptionDetailToMainFile(processInteraction, batchTypeName, batchTypeId);
                });
                return Promise.all(promiseArray_2);
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'GetProcessInteractionsTask: 0');
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            if (result && result.length) {
                var errors = result.filter(function (item) { return item === false; });
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after ProcessInteractionExecute total:', errors.length);
                    return Promise.resolve(false);
                }
            }
            return Promise.resolve(true);
        })
            .then(function (_) {
            var promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessAudio] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, totalNewAudio));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, totalNewAudio));
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.ProcessStudy = function (vcc, processId, studyId, executionCode, currentQueueId) {
        var _this = this;
        var totalStudy = 0;
        var newQueueId = '';
        var studyNluModels = [];
        var studyNluRasaModels = [];
        var billing = new billing_implementation_1.BillingImplementation();
        var processStudy = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetProcessTask(vcc, processId);
        })
            .then(function (result) {
            if (result && result.status === 'FAILED')
                throw 'CANCEL_PROCESS_FAILED';
            if (result && result.status === 'COMPLETE')
                return true;
            throw 'CANCEL_PROCESS_PENDING';
        })
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudy(vcc, studyId);
        })
            .then(function (result) {
            processStudy = result;
            return _this.GetStudyTemplateThemes(vcc, processStudy.templateId);
        })
            .then(function (result) {
            return redis_service_1.Redis.SaveStudyTemplateThemes(vcc, studyId, result);
        })
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudyGenericQueryTask(vcc, processId, studyId);
        })
            .then(function (result) {
            var promiseArray = [];
            if (result && result.length) {
                totalStudy = result.length;
                logger.info('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetStudyGenericQueryTask:', result.length);
                result.forEach(function (studyQuery) {
                    newQueueId = studyQuery.vcc;
                    newQueueId += ':';
                    newQueueId += studyQuery.processId;
                    newQueueId += ':';
                    newQueueId += studyQuery.queryId;
                    newQueueId += ':';
                    newQueueId += 'study';
                    newQueueId += ':';
                    newQueueId += studyQuery.studyId;
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticQuery', newQueueId));
                    billing.AddInsertSemanticAnalysisDetailToMainFile(studyQuery.vcc, 'STUDY', studyQuery.studyId, studyQuery.processId, studyQuery.queryId);
                });
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetStudyGenericQueryTask: 0');
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result && result.length) {
                var errors = result.filter(function (item) { return item === false; });
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after CreateQueueL -> SemanticQuery total:', errors.length);
                    return false;
                }
            }
            return true;
        })
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudyCognitiveInteractionsTask(vcc, processId, studyId);
        })
            .then(function (result) {
            if (!result.status)
                throw 'SERVER_ERROR_GET_INTERACTIONS_TASK_FAILED';
            var processInteractions = result && result.data && result.data.length ? result.data : [];
            if (processInteractions && processInteractions.length) {
                totalStudy += processInteractions.length;
                logger.info('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetStudyCognitiveInteractionsTask:', processInteractions.length);
                var promiseArray_3 = [];
                processInteractions.forEach(function (interaction) {
                    promiseArray_3.push(_this.CognitiveInteractionExecute('study', studyId, interaction, currentQueueId, executionCode));
                    if (studyNluModels.filter(function (n) { return n === interaction.nluId; }).length === 0)
                        studyNluModels.push(interaction.nluId);
                    if (studyNluRasaModels.filter(function (n) { return n === interaction.nluId; }).length === 0 && interaction.cognitiveEngine && interaction.cognitiveEngine === 'RASA')
                        studyNluRasaModels.push(interaction.nluId);
                });
                studyNluModels.map(function (n) {
                    billing.AddInsertCognitiveAnalysisDetailToMainFile(vcc, 'STUDY', studyId, processId, n);
                });
                return Promise.all(promiseArray_3);
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting InteractionCognitive: 0');
                return null;
            }
        })
            .then(function (result) {
            if (result && result.length) {
                var errors = result.filter(function (item) { return item === false; });
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after CreateQueueL -> CognitiveAnalysis total:', errors.length);
                    return false;
                }
            }
            return true;
        })
            .then(function (result) {
            if (studyNluModels.length > 0)
                return _this.RunCurrentBatchesRasaModels(vcc, 'STUDY', studyId, studyNluRasaModels, executionCode, currentQueueId);
            else
                return Promise.resolve(true);
        })
            .then(function (result) {
            var promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessStudy] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, totalStudy));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, totalStudy));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'CANCEL_PROCESS_FAILED') {
                logger.warn('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(true);
            }
            else if (errorMessage === 'CANCEL_PROCESS_PENDING') {
                logger.warn('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(false);
            }
            logger.error('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.ProcessEvaluationPlan = function (vcc, processId, evaluationPlanId, executionCode, currentQueueId) {
        var newQueueId = '';
        var isError = false;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetProcessTask(vcc, processId);
        }).then(function (result) {
            if (result && result.status === 'FAILED')
                throw 'CANCEL_PROCESS_FAILED';
            if (result && result.status === 'COMPLETE')
                return true;
            throw 'CANCEL_PROCESS_PENDING';
        }).then(function (result) {
            if (result) {
                return mssql_service_1.MSSql.GetEvaluationPlanTask(vcc, evaluationPlanId);
            }
            return null;
        }).then(function (result) {
            if (result) {
                newQueueId = vcc;
                newQueueId += ':';
                newQueueId += processId;
                newQueueId += ':';
                newQueueId += result.filterByContentQueryName;
                newQueueId += ':';
                newQueueId += 'evaluationPlan';
                newQueueId += ':';
                newQueueId += evaluationPlanId;
                return redis_service_1.Redis.CreateQueueL('SemanticQuery', newQueueId);
            }
            return null;
        }).then(function (result) {
            var promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessEvaluationPlan] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveRetry(newQueueId, 0));
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, 1));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, 1));
            return Promise.all(promiseArray);
        }).then(function (result) {
            return true;
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            var promiseArray = [];
            if (errorMessage === 'CANCEL_PROCESS_PENDING') {
                isError = true;
                logger.warn('[ProcessImplementation::ProcessEvaluationPlan] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                promiseArray.push(redis_service_1.Redis.DeleteTotalProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteErrorProgress(currentQueueId));
            }
            else {
                throw errorMessage;
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (isError)
                return Promise.resolve(false);
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'CANCEL_PROCESS_FAILED') {
                logger.warn('[ProcessImplementation::ProcessEvaluationPlan] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(true);
            }
            else if (errorMessage === 'CANCEL_PROCESS_PENDING') {
                logger.warn('[ProcessImplementation::ProcessEvaluationPlan] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(false);
            }
            logger.error('[ProcessImplementation::ProcessEvaluationPlan] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.ProcessAbility = function (vcc, processId, evaluationPlanId, executionCode, currentQueueId) {
        var _this = this;
        var newQueueId = '';
        var processEvaluationPlan = null;
        var totalAbility = 0;
        var isError = false;
        var evaluationPlanNluModels = [];
        var evaluationPlanNluRasaModels = [];
        var billing = new billing_implementation_1.BillingImplementation();
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetProcessEvaluationPlanTask(vcc, processId, evaluationPlanId);
        }).then(function (result) {
            if (result && result.status === 'FAILED') {
                logger.error('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' Process failed, canceling evaluation phase');
                throw 'CANCEL_PROCESS_FAILED';
            }
            processEvaluationPlan = result;
            if (processEvaluationPlan
                && processEvaluationPlan.status === 'COMPLETE'
                && processEvaluationPlan.evaluationPlanStatus === 5
                && (processEvaluationPlan.evaluationPlanType === 'A'
                    ||
                        processEvaluationPlan.evaluationPlanType === 'X'))
                return true;
            throw 'CANCEL_PROCESS_PENDING';
        })
            .then(function (result) {
            return _this.GetEvaluationPlanAbilities(vcc, processEvaluationPlan.evaluationFormatId);
        })
            .then(function (result) {
            return redis_service_1.Redis.SaveEvaluationPlanAbilities(vcc, processEvaluationPlan.evaluationPlanId, result);
        })
            .then(function (result) {
            if (result) {
                return mssql_service_1.MSSql.GetEvaluationPlanAbilityGenericQueryTask(vcc, evaluationPlanId);
            }
            return null;
        }).then(function (result) {
            var promiseArray = [];
            if (result && result.length) {
                totalAbility = result.length;
                logger.info('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetEvaluationPlanAbilityGenericQueryTask:', result.length);
                result.forEach(function (query) {
                    newQueueId = vcc;
                    newQueueId += ':';
                    newQueueId += processId;
                    newQueueId += ':';
                    newQueueId += query.name;
                    newQueueId += ':';
                    newQueueId += 'ability';
                    newQueueId += ':';
                    newQueueId += evaluationPlanId;
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticQuery', newQueueId));
                    promiseArray.push(redis_service_1.Redis.SaveRetry(newQueueId, 0));
                    billing.AddInsertSemanticAnalysisDetailToMainFile(vcc, 'EVALUATION_PLAN', evaluationPlanId, processId, query.name);
                });
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (result && result.length) {
                var errors = result.filter(function (item) { return item === false; });
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after CreateQueueL -> SemanticQuery total:', errors.length);
                    return false;
                }
            }
            return true;
        }).then(function (result) {
            return mssql_service_1.MSSql.GetEvaluationPlanCognitiveInteractionTask(vcc, evaluationPlanId);
        }).then(function (result) {
            if (!result.status)
                throw 'SERVER_ERROR_GET_INTERACTIONS_TASK_FAILED';
            var processInteractions = result && result.data && result.data.length ? result.data : [];
            if (processInteractions && processInteractions.length) {
                totalAbility += processInteractions.length;
                logger.info('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetEvaluationPlanCognitiveInteractionTask:', processInteractions.length);
                var promiseArray_4 = [];
                processInteractions.forEach(function (interaction) {
                    promiseArray_4.push(_this.CognitiveInteractionExecute('ability', evaluationPlanId, interaction, currentQueueId, executionCode));
                    if (evaluationPlanNluModels.filter(function (n) { return n === interaction.nluId; }).length === 0)
                        evaluationPlanNluModels.push(interaction.nluId);
                    if (evaluationPlanNluRasaModels.filter(function (n) { return n === interaction.nluId; }).length === 0 && interaction.cognitiveEngine && interaction.cognitiveEngine === 'RASA')
                        evaluationPlanNluRasaModels.push(interaction.nluId);
                });
                evaluationPlanNluModels.map(function (n) {
                    billing.AddInsertCognitiveAnalysisDetailToMainFile(vcc, 'EVALUATION_PLAN', evaluationPlanId, processId, n);
                });
                return Promise.all(promiseArray_4);
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetEvaluationPlanCognitiveInteractionTask: 0');
                return null;
            }
        }).then(function (result) {
            if (result && result.length) {
                var errors = result.filter(function (item) { return item === false; });
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after InteractionProcessExecute total:', errors.length);
                    return false;
                }
            }
            return true;
        }).then(function (result) {
            if (evaluationPlanNluModels.length > 0)
                return _this.RunCurrentBatchesRasaModels(vcc, 'EVALUATION_PLAN', evaluationPlanId, evaluationPlanNluRasaModels, executionCode, currentQueueId);
            else
                return Promise.resolve(true);
        }).then(function (result) {
            var promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessAbility] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, totalAbility));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, totalAbility));
            return Promise.all(promiseArray);
        }).then(function (result) {
            return true;
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            var promiseArray = [];
            if (errorMessage === 'CANCEL_PROCESS_PENDING') {
                isError = true;
                logger.warn('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                promiseArray.push(redis_service_1.Redis.DeleteTotalProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteErrorProgress(currentQueueId));
            }
            else {
                throw errorMessage;
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (isError)
                return Promise.resolve(false);
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'CANCEL_PROCESS_FAILED') {
                logger.warn('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(true);
            }
            else if (errorMessage === 'CANCEL_PROCESS_PENDING') {
                logger.warn('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(false);
            }
            logger.error('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.ProcessI6UpdateExecute = function (interactionToUpdate, queueId, executionCode) {
        var _this = this;
        var currentKey = '';
        var itemExist = false;
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!interactionToUpdate)
                throw 'INTERACTION_IS_NOT_FOUND';
            currentKey = _this.GetQueueKeyFromProcessInteraction(interactionToUpdate, 'interaction');
            return redis_service_1.Redis.GetProcessI6Update(queueId, currentKey);
        })
            .then(function (result) {
            if (!result) {
                return redis_service_1.Redis.SaveProcessI6Update(queueId, currentKey, interactionToUpdate);
            }
            itemExist = true;
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (!itemExist && result) {
                return redis_service_1.Redis.CreateQueueL('I6Update', currentKey);
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (!result) {
                if (itemExist) {
                    logger.warn('[ProcessImplementation::ProcessI6UpdateExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'interaction exists, queue process was not inserted from the Interactions queue');
                    return Promise.resolve(true);
                }
                logger.error('[ProcessImplementation::ProcessI6UpdateExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'queue process was not inserted from the Interactions queue');
                return Promise.resolve(false);
            }
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessI6UpdateExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.ProcessInteractionExecute = function (processInteraction, queueId, executionCode) {
        var _this = this;
        var currentKey = '';
        var itemExist = false;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!processInteraction)
                throw 'AUDIO_IS_NOT_FOUND';
            currentKey = _this.GetQueueKeyFromProcessInteraction(processInteraction);
            return redis_service_1.Redis.GetProcessInteraction(queueId, currentKey);
        }).then(function (result) {
            if (!result) {
                return redis_service_1.Redis.SaveProcessInteraction(queueId, currentKey, processInteraction);
            }
            itemExist = true;
            return null;
        }).then(function (result) {
            if (!itemExist && result) {
                return redis_service_1.Redis.CreateQueueL('Audios', currentKey);
            }
            return null;
        }).then(function (result) {
            if (!result) {
                if (itemExist) {
                    logger.warn('[ProcessImplementation::ProcessInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'audio exists, queue process was not inserted from the Audios queue');
                    return Promise.resolve(true);
                }
                logger.error('[ProcessImplementation::ProcessInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'queue process was not inserted from the Audios queue');
                return Promise.resolve(false);
            }
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.CognitiveInteractionExecute = function (type, typeId, interaction, queueId, executionCode) {
        var _this = this;
        var currentKey = '';
        var itemExist = false;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!interaction)
                throw 'INTERACTION_COGNITIVE_NOT_FOUND';
            currentKey = _this.GetQueueKeyFromProcessInteraction(interaction);
            currentKey += ':';
            currentKey += interaction.nluId;
            currentKey += ':';
            currentKey += type;
            currentKey += ':';
            currentKey += typeId;
            return redis_service_1.Redis.GetCognitiveInteraction(currentKey);
        }).then(function (result) {
            if (!result) {
                interaction.statusDescription = 'pendingPhrases';
                return redis_service_1.Redis.SaveCognitiveInteraction(currentKey, interaction);
            }
            itemExist = true;
            return null;
        }).then(function (result) {
            if (!itemExist && result) {
                return redis_service_1.Redis.CreateQueueL('CognitiveAnalysis', currentKey);
            }
            return null;
        }).then(function (result) {
            if (!result) {
                if (itemExist) {
                    logger.warn('[ProcessImplementation::CognitiveInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'audio exists, queue process was not inserted from the Audios queue');
                    return Promise.resolve(true);
                }
                logger.error('[ProcessImplementation::CognitiveInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'queue process was not inserted from the Audios queue');
                return Promise.resolve(false);
            }
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::CognitiveInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    ProcessImplementation.prototype.GetStudyTemplateThemes = function (vcc, templateId) {
        var templateTemplateThemes = [];
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetTemplateThemes(vcc, templateId);
        })
            .then(function (result) {
            templateTemplateThemes = result;
            var promiseArray = [];
            templateTemplateThemes.map(function (ttt) {
                promiseArray.push(template_service_1.GetTemplateItems(vcc, ttt.id, true));
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            result.map(function (r, i) {
                templateTemplateThemes[i].items = r;
            });
            return Promise.resolve(templateTemplateThemes);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_GET_STUDY_THEMES_FAILED';
            logger.error('[ProcessImplementation::GetStudyTemplateThemes] Error: ' + JSON.stringify(error));
            return Promise.reject(errorMessage);
        });
    };
    ProcessImplementation.prototype.GetEvaluationPlanAbilities = function (vcc, evaluationFormatId) {
        var evaluationFormatAbilities = [];
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAbilityEvaluationFormats(vcc, evaluationFormatId);
        })
            .then(function (result) {
            evaluationFormatAbilities = result;
            var promiseArray = [];
            evaluationFormatAbilities.map(function (efa) {
                promiseArray.push(ability_service_1.GetAbilityQueries(vcc, efa.idAbility, true));
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            result.map(function (r, i) {
                evaluationFormatAbilities[i].queries = r;
            });
            return Promise.resolve(evaluationFormatAbilities);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_GET_EVALUATIONPLAN_ABILITIES_FAILED';
            logger.error('[ProcessImplementation::GetEvaluationPlanAbilities] Error: ' + JSON.stringify(error));
            return Promise.reject(errorMessage);
        });
    };
    ProcessImplementation.prototype.GetQueueKeyFromProcessInteraction = function (interaction, keyType) {
        if (keyType === void 0) { keyType = 'detail'; }
        if (!interaction)
            throw 'INTERACTION_NOT_FOUND';
        var currentKey = '';
        var interactionId = '';
        if (interaction.interactionType === 'CALL') {
            interactionId = JSON.parse(JSON.stringify(interaction.id));
        }
        else {
            interactionId = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + JSON.parse(JSON.stringify(interaction.id));
        }
        currentKey = interaction.vcc;
        currentKey += ':';
        currentKey += interaction.processId;
        currentKey += ':';
        currentKey += interaction.interactionType;
        currentKey += ':';
        currentKey += interactionId.split(':').join('|');
        if (keyType === 'detail') {
            currentKey += ':';
            currentKey += interaction.section ? interaction.section : '0';
            currentKey += ':';
            currentKey += interaction.part ? interaction.part : '0';
        }
        return currentKey;
    };
    ProcessImplementation.prototype.RunCurrentBatchesRasaModels = function (vcc, batchTypeName, batchTypeId, nlus, executionCode, currentQueueId) {
        return promises_shared_1.DummyPromise().then(function (result) {
            var appIds = [];
            nlus.map(function (n) { return appIds.push(n = vcc + '::' + n); });
            return redis_service_1.Redis.SaveCurrentBatchesRasaModels(batchTypeName + '::' + batchTypeId, appIds);
        }).then(function (result) {
            logger.info('[ProcessImplementation::RunCurrentBatchesRasaModels] Result: ' + result);
            return Promise.resolve(true);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
            logger.error('[ProcessImplementation::RunCurrentBatchesRasaModels] Error: ' + JSON.stringify(error));
            return Promise.reject(errorMessage);
        });
    };
    return ProcessImplementation;
}());
exports.ProcessImplementation = ProcessImplementation;
